class Grid

  def unit(num)
    # 4=m,3=cm,2=mm,1=feet,0=inches
    @unit = Sketchup.active_model.options["UnitsOptions"]["LengthUnit"]
    
    case
    when @unit == 0
      num.inch
    when @unit == 1
      num.feet
    when @unit == 2
      num.mm
    when @unit == 3
      num.cm
    when @unit == 4
      num.m
    else
    end    
   
  end
 
 
  def create_3d_grid
  
    model = Sketchup.active_model
    model.start_operation "Create 2D Grid"
    entities = model.active_entities
    
    definitions_list = model.definitions
    grid_2d_comp_def = definitions_list.add("2D Grid")
    
   # Prompts the user to insert values and defines default values.
      prompts = ["Interval between axes (along X)", "Interval between axes (along Y)", "Height between levels/stories"]
      def_values = ["8.0;13.0;6.0", "12.0;15.0", "5.0"]
      results = inputbox prompts, def_values, "Grid parameters"
      interval_x_str, interval_y_str, interval_z_str = results
   
    if interval_x_str == nil or interval_y_str == nil or interval_z_str == nil
    
    return UI.messagebox("Insert values.")
        
    end
    
    intervals_x = []
    intervals_y = []
    intervals_z = []
    
    # Writes floats from intervals strings to intervals arrays, the pattern is ; . 
    intervals_x = interval_x_str.split(/\;/).map {|int| Float(int)}
    intervals_y = interval_y_str.split(/\;/).map {|int| Float(int)}
    intervals_z = interval_z_str.split(/\;/).map {|int| Float(int)}
    
    # Checks if each array element is smaller or equal to 0.0 and if any is, pops up a message and end the method.
    if intervals_x.each {|int| int <= 0.0} or intervals_y.each {|int| int <= 0.0} or intervals_z.each {|int| int <= 0.0} then
    else
    return UI.messagebox("Values must be positive.")
    end
    
    nu_x = intervals_x.length
    nu_y = intervals_y.length
    nu_z = intervals_z.length
    
    
    # Scales the intervals to user-used units and calculates the dimensions of a cube which contains the 3D grid.
    n = 0
    length_x = 0
    intervals_x.each do |interval_x|
    intervals_x[n] = unit(interval_x)
    length_x = length_x+intervals_x[n]
    n += 1
    end
        
    n = 0
    length_y = 0
    intervals_y.each do |interval_y|
    intervals_y[n] = unit(interval_y)
    length_y = length_y+intervals_y[n]
    n += 1
    end
    
    n = 0
    length_z = 0
    intervals_z.each do |interval_z|
    intervals_z[n] = unit(interval_z)
    length_z = length_z+intervals_z[n]
    n += 1
    end
 
    pts = []
    step = -1
    x = 0.0
    y = 0.0
    
    # Finds start and finish points for edges and draws them into a component definition grid_2d_comp_def.
    while x < length_x
      if step == -1 then
      pts[0] = Geom::Point3d.new(0, 0, 0)
      pts[1] = Geom::Point3d.new(0, length_y, 0)
      else
      x = x+intervals_x[step]
      pts[0] = Geom::Point3d.new(x, 0, 0)
      pts[1] = Geom::Point3d.new(x, length_y, 0)
      end
      line_y = grid_2d_comp_def.entities.add_edges(pts[0], pts[1])
      step += 1
    end
    
    step = -1
     
    while y < length_y
      if step == -1 then
      pts[0] = Geom::Point3d.new(0, 0, 0)
      pts[1] = Geom::Point3d.new(length_x, 0, 0)
      else
      y = y+intervals_y[step]
      pts[0] = Geom::Point3d.new(0, y, 0)
      pts[1] = Geom::Point3d.new(length_x, y, 0)
      end
      line_y = grid_2d_comp_def.entities.add_edges(pts[0], pts[1])
      step += 1
    end
   
    model.commit_operation
     
    model.start_operation "Duplicate 2D grid"
    
    step = -1
    z = 0.0
    
    # Positions grid_2d_comp_def instances on different heights.
    while z < length_z
      if step == -1 then
      grid_2d_instance = entities.add_instance grid_2d_comp_def, [0, 0, 0]
      else
      z = z+intervals_z[step]
      grid_2d_instance = entities.add_instance grid_2d_comp_def, [0, 0, z]
      end
      step += 1
    end
    
    model.commit_operation
    
    # Zooms to include the entire model (grid).
    view = Sketchup.active_model.active_view
    view_all = view.zoom_extents
    
  end

end
