require "Point on edge.rb"

class Grid

  # Takes the @intersection_points array and positions the chosen component at all the array points.
  def position_at_intersection(intersection_points)

    # Opens up an explorer panel and lets you choose the component definition and saves its path.
    component_path = UI.openpanel("Open Component File", "#{$main_path}/DC Library", "*.skp")
    component_def = Sketchup.active_model.definitions.load(component_path)
    
    intersection_points.each do |point|
      Sketchup.active_model.active_entities.add_instance(component_def, point)
    end
    
  end

  # Gets edges array, the number of edges n_end and the instance in which the edges are located. Processes them depending
  # on the instance class, finds points where edges intersect and writes these points into @intersection_points.
  def find_intersection_points(edges, n_end, instance)
    
    intersection_points = []
  
    # Takes one edge at a time and finds intersections with all other lines.
    edges.each do |current_edge|
      n = 0
      i = edges.index(current_edge)
      
      until n == n_end do
      
        # Checks if the current_edge is different from the edge it should intersect.
        # If they are the same edge, does nothing, else finds their intersection point.
        if i == n  then
          # Do nothing.
        else
          intersection_point = Geom.intersect_line_line(current_edge.line, edges[n].line)
          
          # Checks if such an intersection already exists or if there is no intersection at all (nil).
          # If its a new, not-nil intersection, pushes it into the intersection_points array.
          if (intersection_points.any? {|previous_point| previous_point == intersection_point} == true or intersection_point == nil)  then
            # Do nothing
          else
          
            # Checks if intersection_point is on current_edge and on edges[n], which are the two addressed edges, with
            # on_edge? method from Point on edge.rb.
            # If the point is on both edges, writes it into intersection_point array.
            if intersection_point.on_edge?(current_edge) == true and intersection_point.on_edge?(edges[n]) == true then
              intersection_points.push intersection_point
            else             
              # Do nothing
            end
            
          end
                  
        end
      
        n += 1
     
      end
    
    end
    
    # Examines the instance's class and if it's a Group or a ComponentInstance, moves/transforms all the points with the
    # trans_vector.inverse vector. trans_vector is a vector from the group/component instance origin to the global model origin. 
    intersection_points.each do |point|
      case
      when instance.class == Sketchup::Group
        trans_vector = Geom::Transformation.translation(instance.transformation.origin.vector_to(ORIGIN))
        point_trans = point.transform(trans_vector.inverse)
        
        @intersection_points.push point_trans
        
      when instance.class == Sketchup::ComponentInstance
        trans_vector = Geom::Transformation.translation(instance.transformation.origin.vector_to(ORIGIN))
        point_trans = point.transform(trans_vector.inverse)
        
        @intersection_points.push point_trans
        
      else
        @intersection_points.push point
      end
    end
        
  end

  
  # Goes through the entire everything array, defines each instance's class and sends its edges to find_intersection_points.
  # After find_intersection_points processes the edges this method recieves the @intersection_points array and informs the user
  # if there are no edges in the selection or if these edges don't intersect.
  def find_edges_and_points(everything)
  
    # Defines all the necessary arrays which are used in this plugin.
    normal_edges = []
    normal_edges_n = 0
    group_edges = []
    group_edges_n = 0
    component_edges = []
    component_edges_n = 0
    
    @intersection_points = []
    
    # Goes through the entire everything array and processes each instance according to its class.
    # Regardless the class every single edge array is sent to find_intersection_points.
    everything.each do |instance|
      case 
      when instance.class == Sketchup::Edge      
        normal_edges.push instance     
        normal_edges_n += 1
      
      when instance.class == Sketchup::Group
        group_entities = instance.entities
        
        group_edges = []
        group_edges_n = 0
        
        group_entities.each do |group_entity|
          if group_entity.class == Sketchup::Edge      
            group_edges.push group_entity     
            group_edges_n += 1
          end
        end
        # Sends all the edges from this group to find_intersection_points.
        find_intersection_points(group_edges, group_edges_n, instance)
              
      when instance.class == Sketchup::ComponentInstance
        component_entities = instance.definition.entities
        
        component_edges = []
        component_edges_n = 0
        
        component_entities.each do |component_entity|
          if component_entity.class == Sketchup::Edge      
            component_edges.push component_entity     
            component_edges_n += 1
          end
        end
        # Sends all the edges from this component instance to find_intersection_points.
        find_intersection_points(component_edges, component_edges_n, instance)
       
      end
        
    end

    # If no edges were selected, pops up messagebox and ends find_edges_and_points method.
    if normal_edges.empty? and group_edges.empty? and component_edges.empty?
      return UI.messagebox("No edges were selected.")
    end
   
    # Sends all the "normal" edges in the model to find_intersection_points.
    find_intersection_points(normal_edges, normal_edges_n, nil)
    
    
    # If no edges in the model/same group/same component instance intersect, pops up messagebox and ends find_edges_and_points method.
    if @intersection_points.empty?
      return UI.messagebox("Edges do not intersect.")
    end

  end
 
 
  # Sends selection_everything to find_edges_and_points, recieves @intersection_points array from it and sends
  # it to position_at_intersection.
  def grid_positioning(selection_everything)
  
    find_edges_and_points(selection_everything)
    
    position_at_intersection(@intersection_points)
    
  end
  
end